@@ -27,6 +27,7 @@ VERIFY_LEVEL = 2
 OCSP_ENABLED = False
 DEFAULT_REGION = "us-east-1"
 STUNNEL_LOGS_FILE = "/var/log/amazon/efs/%s.stunnel.log" % FS_ID
+PROXY_LOGS_FILE = "/var/log/amazon/efs/%s.efs-proxy.log" % FS_ID
 
 
 def _get_config(
@@ -41,7 +42,6 @@ def _get_config(
     stunnel_libwrap_option_supported=True,
     fips_mode_enabled=False,
 ):
-
     options = []
     if stunnel_check_cert_hostname_supported:
         options.append(b"checkHost              = peer certificate host name pattern")
@@ -99,7 +99,12 @@ def _get_config(
     return config
 
 
-def _get_mount_options(port=PORT):
+def _get_mount_options_tls(port=PORT):
+    options = {"tlsport": port, "tls": None}
+    return options
+
+
+def _get_mount_options_non_tls(port=PORT):
     options = {
         "tlsport": port,
     }
@@ -153,7 +158,7 @@ def _validate_config(stunnel_config_file, expected_global_config, expected_efs_c
     assert expected_efs_config == actual_efs_config
 
 
-def _get_expected_efs_config(
+def _get_expected_efs_config_tls(
     port=PORT,
     dns_name=DNS_NAME,
     verify=mount_efs.DEFAULT_STUNNEL_VERIFY_LEVEL,
@@ -162,8 +167,8 @@ def _get_expected_efs_config(
     check_cert_validity=False,
     disable_libwrap=True,
     fallback_ip_address=None,
+    efs_proxy_enabled=True,
 ):
-
     expected_efs_config = dict(mount_efs.STUNNEL_EFS_CONFIG)
     expected_efs_config["accept"] = expected_efs_config["accept"] % port
     if not fallback_ip_address:
@@ -172,28 +177,50 @@ def _get_expected_efs_config(
         expected_efs_config["connect"] = (
             expected_efs_config["connect"] % fallback_ip_address
         )
+
     expected_efs_config["verify"] = str(verify)
 
-    if check_cert_hostname:
+    if check_cert_hostname or efs_proxy_enabled:
         expected_efs_config["checkHost"] = dns_name[dns_name.index(FS_ID) :]
 
-    if check_cert_validity and ocsp_override:
+    if not efs_proxy_enabled and mount_efs.is_ipv6_address(fallback_ip_address):
+        expected_efs_config["sni"] = dns_name[dns_name.index(FS_ID) :]
+
+    if check_cert_validity and ocsp_override and (not efs_proxy_enabled):
         expected_efs_config["OCSPaia"] = "yes"
 
-    if disable_libwrap:
+    if disable_libwrap and (not efs_proxy_enabled):
         expected_efs_config["libwrap"] = "no"
 
     return expected_efs_config
 
 
-def _test_check_cert_hostname(
+def _get_expected_efs_config_non_tls(
+    port=PORT,
+    dns_name=DNS_NAME,
+    fallback_ip_address=None,
+):
+    expected_efs_config = dict(mount_efs.STUNNEL_EFS_CONFIG)
+    expected_efs_config["accept"] = expected_efs_config["accept"] % port
+    if not fallback_ip_address:
+        expected_efs_config["connect"] = expected_efs_config["connect"] % dns_name
+    else:
+        expected_efs_config["connect"] = (
+            expected_efs_config["connect"] % fallback_ip_address
+        )
+
+    return expected_efs_config
+
+
+# Check the hostname behavior when using stunnel instead of efs-proxy.
+def _test_check_cert_hostname_stunnel(
     mocker,
     tmpdir,
     stunnel_check_cert_hostname_supported,
     stunnel_check_cert_hostname,
     expected_check_cert_hostname_config_value,
 ):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
     config_file = mount_efs.write_stunnel_config_file(
         _get_config(
@@ -208,8 +235,9 @@ def _test_check_cert_hostname(
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=False,
     )
 
     utils.assert_called_once(ca_mocker)
@@ -217,8 +245,9 @@ def _test_check_cert_hostname(
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(
-            check_cert_hostname=expected_check_cert_hostname_config_value
+        _get_expected_efs_config_tls(
+            check_cert_hostname=expected_check_cert_hostname_config_value,
+            efs_proxy_enabled=False,
         ),
     )
 
@@ -230,7 +259,7 @@ def _test_check_cert_validity(
     stunnel_check_cert_validity,
     expected_check_cert_validity_config_value,
 ):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
     config_file = mount_efs.write_stunnel_config_file(
         _get_config(
@@ -244,8 +273,9 @@ def _test_check_cert_validity(
         DNS_NAME,
         VERIFY_LEVEL,
         stunnel_check_cert_validity,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
 
     utils.assert_called_once(ca_mocker)
@@ -253,14 +283,14 @@ def _test_check_cert_validity(
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(
+        _get_expected_efs_config_tls(
             check_cert_validity=expected_check_cert_validity_config_value
         ),
     )
 
 
 def test_write_stunnel_config_file(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -272,20 +302,21 @@ def test_write_stunnel_config_file(mocker, tmpdir):
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
     utils.assert_called_once(ca_mocker)
 
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(),
+        _get_expected_efs_config_tls(),
     )
 
 
 def test_write_stunnel_config_file_with_az_as_dns_name(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -297,15 +328,16 @@ def test_write_stunnel_config_file_with_az_as_dns_name(mocker, tmpdir):
         DNS_NAME_WITH_AZ,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
     utils.assert_called_once(ca_mocker)
 
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(dns_name=DNS_NAME_WITH_AZ),
+        _get_expected_efs_config_tls(dns_name=DNS_NAME_WITH_AZ),
     )
 
 
@@ -315,7 +347,7 @@ def _test_enable_disable_libwrap(
     system_release="unknown",
     libwrap_supported=True,
 ):
-    mocker.patch("mount_efs.add_stunnel_ca_options")
+    mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
     ver_mocker = mocker.patch(
         "mount_efs.get_system_release_version", return_value=system_release
@@ -330,20 +362,21 @@ def _test_enable_disable_libwrap(
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
 
     utils.assert_called_once(ver_mocker)
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(disable_libwrap=libwrap_supported),
+        _get_expected_efs_config_tls(disable_libwrap=libwrap_supported),
     )
 
 
 def test_write_stunnel_config_with_debug(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -355,8 +388,9 @@ def test_write_stunnel_config_with_debug(mocker, tmpdir):
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
     utils.assert_called_once(ca_mocker)
 
@@ -366,19 +400,21 @@ def test_write_stunnel_config_with_debug(mocker, tmpdir):
     expected_global_config["debug"] = "debug"
     expected_global_config["output"] = os.path.join(
         mount_efs.LOG_DIR,
-        "%s.stunnel.log"
+        "%s.efs-proxy.log"
         % mount_efs.get_mount_specific_filename(FS_ID, MOUNT_POINT, PORT),
     )
 
-    _validate_config(config_file, expected_global_config, _get_expected_efs_config())
+    _validate_config(
+        config_file, expected_global_config, _get_expected_efs_config_tls()
+    )
 
 
 def test_write_stunnel_config_with_debug_and_logs_file(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
     config_file = mount_efs.write_stunnel_config_file(
         _get_config(
-            mocker, stunnel_debug_enabled=True, stunnel_logs_file=STUNNEL_LOGS_FILE
+            mocker, stunnel_debug_enabled=True, stunnel_logs_file=PROXY_LOGS_FILE
         ),
         state_file_dir,
         FS_ID,
@@ -387,8 +423,9 @@ def test_write_stunnel_config_with_debug_and_logs_file(mocker, tmpdir):
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=True,
     )
     utils.assert_called_once(ca_mocker)
 
@@ -396,15 +433,48 @@ def test_write_stunnel_config_with_debug_and_logs_file(mocker, tmpdir):
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir)
     )
     expected_global_config["debug"] = "debug"
-    expected_global_config["output"] = STUNNEL_LOGS_FILE
+    expected_global_config["output"] = PROXY_LOGS_FILE
 
-    _validate_config(config_file, expected_global_config, _get_expected_efs_config())
+    _validate_config(
+        config_file, expected_global_config, _get_expected_efs_config_tls()
+    )
+
+
+# We should always write "checkHost" into the stunnel config when using efs-proxy for TLS mounts.
+def test_write_stunnel_config_efs_proxy_check_cert_hostname_tls(mocker, tmpdir):
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
+    supported_opt_mock = mocker.patch("mount_efs.is_stunnel_option_supported")
+    state_file_dir = str(tmpdir)
+    config_file = mount_efs.write_stunnel_config_file(
+        _get_config(mocker),
+        state_file_dir,
+        FS_ID,
+        MOUNT_POINT,
+        PORT,
+        DNS_NAME,
+        VERIFY_LEVEL,
+        OCSP_ENABLED,
+        _get_mount_options_tls(),
+        DEFAULT_REGION,
+        efs_proxy_enabled=True,
+    )
+
+    utils.assert_called_once(ca_mocker)
+    utils.assert_not_called(supported_opt_mock)
+
+    _validate_config(
+        config_file,
+        _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
+        _get_expected_efs_config_tls(
+            efs_proxy_enabled=True,
+        ),
+    )
 
 
 def test_write_stunnel_config_check_cert_hostname_supported_flag_not_set(
     mocker, tmpdir
 ):
-    _test_check_cert_hostname(
+    _test_check_cert_hostname_stunnel(
         mocker,
         tmpdir,
         stunnel_check_cert_hostname_supported=True,
@@ -416,7 +486,7 @@ def test_write_stunnel_config_check_cert_hostname_supported_flag_not_set(
 def test_write_stunnel_config_check_cert_hostname_supported_flag_set_false(
     mocker, capsys, tmpdir
 ):
-    _test_check_cert_hostname(
+    _test_check_cert_hostname_stunnel(
         mocker,
         tmpdir,
         stunnel_check_cert_hostname_supported=True,
@@ -428,7 +498,7 @@ def test_write_stunnel_config_check_cert_hostname_supported_flag_set_false(
 def test_write_stunnel_config_check_cert_hostname_supported_flag_set_true(
     mocker, tmpdir
 ):
-    _test_check_cert_hostname(
+    _test_check_cert_hostname_stunnel(
         mocker,
         tmpdir,
         stunnel_check_cert_hostname_supported=True,
@@ -440,7 +510,7 @@ def test_write_stunnel_config_check_cert_hostname_supported_flag_set_true(
 def test_write_stunnel_config_check_cert_hostname_not_supported_flag_not_specified(
     mocker, capsys, tmpdir
 ):
-    _test_check_cert_hostname(
+    _test_check_cert_hostname_stunnel(
         mocker,
         tmpdir,
         stunnel_check_cert_hostname_supported=False,
@@ -452,7 +522,7 @@ def test_write_stunnel_config_check_cert_hostname_not_supported_flag_not_specifi
 def test_write_stunnel_config_check_cert_hostname_not_supported_flag_set_false(
     mocker, capsys, tmpdir
 ):
-    _test_check_cert_hostname(
+    _test_check_cert_hostname_stunnel(
         mocker,
         tmpdir,
         stunnel_check_cert_hostname_supported=False,
@@ -464,7 +534,7 @@ def test_write_stunnel_config_check_cert_hostname_not_supported_flag_set_false(
 def test_write_stunnel_config_check_cert_hostname_not_supported_flag_set_true(
     mocker, capsys, tmpdir
 ):
-    mocker.patch("mount_efs.add_stunnel_ca_options")
+    mocker.patch("mount_efs.add_tunnel_ca_options")
 
     with pytest.raises(SystemExit) as ex:
         mount_efs.write_stunnel_config_file(
@@ -480,8 +550,9 @@ def test_write_stunnel_config_check_cert_hostname_not_supported_flag_set_true(
             DNS_NAME,
             VERIFY_LEVEL,
             OCSP_ENABLED,
-            _get_mount_options(),
+            _get_mount_options_tls(),
             DEFAULT_REGION,
+            efs_proxy_enabled=False,
         )
 
     assert 0 != ex.value.code
@@ -530,7 +601,7 @@ def test_write_stunnel_config_check_cert_validity_not_supported_ocsp_disabled(
 def test_write_stunnel_config_check_cert_validity_not_supported_ocsp_enabled(
     mocker, capsys, tmpdir
 ):
-    mocker.patch("mount_efs.add_stunnel_ca_options")
+    mocker.patch("mount_efs.add_tunnel_ca_options")
 
     with pytest.raises(SystemExit) as ex:
         mount_efs.write_stunnel_config_file(
@@ -546,8 +617,9 @@ def test_write_stunnel_config_check_cert_validity_not_supported_ocsp_enabled(
             DNS_NAME,
             VERIFY_LEVEL,
             True,
-            _get_mount_options(),
+            _get_mount_options_tls(),
             DEFAULT_REGION,
+            efs_proxy_enabled=False,
         )
 
     assert 0 != ex.value.code
@@ -558,7 +630,7 @@ def test_write_stunnel_config_check_cert_validity_not_supported_ocsp_enabled(
 
 
 def test_write_stunnel_config_with_verify_level(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
     verify = 0
     config_file = mount_efs.write_stunnel_config_file(
@@ -570,7 +642,7 @@ def test_write_stunnel_config_with_verify_level(mocker, tmpdir):
         DNS_NAME,
         verify,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
     )
     utils.assert_not_called(ca_mocker)
@@ -578,7 +650,7 @@ def test_write_stunnel_config_with_verify_level(mocker, tmpdir):
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(check_cert_validity=False, verify=verify),
+        _get_expected_efs_config_tls(check_cert_validity=False, verify=verify),
     )
 
 
@@ -591,7 +663,7 @@ def test_write_stunnel_config_libwrap_supported(mocker, tmpdir):
 
 
 def test_write_stunnel_config_with_fall_back_ip_address(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -603,7 +675,7 @@ def test_write_stunnel_config_with_fall_back_ip_address(mocker, tmpdir):
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
         fallback_ip_address=FALLBACK_IP_ADDRESS,
     )
@@ -613,7 +685,7 @@ def test_write_stunnel_config_with_fall_back_ip_address(mocker, tmpdir):
     _validate_config(
         config_file,
         _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
-        _get_expected_efs_config(fallback_ip_address=FALLBACK_IP_ADDRESS),
+        _get_expected_efs_config_tls(fallback_ip_address=FALLBACK_IP_ADDRESS),
     )
 
 
@@ -635,10 +707,16 @@ def test_write_stunnel_config_foreground_quiet_supported_debug_enabled(mocker, t
     )
 
 
+def test_write_stunnel_config_foreground_quiet_supported_debug_enabled(mocker, tmpdir):
+    _test_stunnel_config_foreground_quiet_helper(
+        mocker, tmpdir, foreground_quiet_supported=True, stunnel_debug_enabled=True
+    )
+
+
 def _test_stunnel_config_foreground_quiet_helper(
     mocker, tmpdir, foreground_quiet_supported, stunnel_debug_enabled
 ):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -654,8 +732,9 @@ def _test_stunnel_config_foreground_quiet_helper(
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
+        efs_proxy_enabled=False,
     )
     utils.assert_called_once(ca_mocker)
 
@@ -672,11 +751,15 @@ def _test_stunnel_config_foreground_quiet_helper(
             "%s.stunnel.log"
             % mount_efs.get_mount_specific_filename(FS_ID, MOUNT_POINT, PORT),
         )
-    _validate_config(config_file, expected_global_config, _get_expected_efs_config())
+    _validate_config(
+        config_file,
+        expected_global_config,
+        _get_expected_efs_config_tls(efs_proxy_enabled=False),
+    )
 
 
 def test_write_stunnel_config_fips_enabled(mocker, tmpdir):
-    ca_mocker = mocker.patch("mount_efs.add_stunnel_ca_options")
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
     state_file_dir = str(tmpdir)
 
     config_file = mount_efs.write_stunnel_config_file(
@@ -688,7 +771,7 @@ def test_write_stunnel_config_fips_enabled(mocker, tmpdir):
         DNS_NAME,
         VERIFY_LEVEL,
         OCSP_ENABLED,
-        _get_mount_options(),
+        _get_mount_options_tls(),
         DEFAULT_REGION,
     )
     utils.assert_called_once(ca_mocker)
@@ -701,5 +784,69 @@ def test_write_stunnel_config_fips_enabled(mocker, tmpdir):
     _validate_config(
         config_file,
         expected_global_config,
-        _get_expected_efs_config(),
+        _get_expected_efs_config_tls(),
+    )
+
+
+def test_non_tls_mount_with_proxy(mocker, tmpdir):
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
+    state_file_dir = str(tmpdir)
+
+    config_file = mount_efs.write_stunnel_config_file(
+        _get_config(mocker),
+        state_file_dir,
+        FS_ID,
+        MOUNT_POINT,
+        PORT,
+        DNS_NAME,
+        VERIFY_LEVEL,
+        OCSP_ENABLED,
+        _get_mount_options_non_tls(),
+        DEFAULT_REGION,
+        efs_proxy_enabled=True,
+    )
+    utils.assert_not_called(ca_mocker)
+
+    expected_global_config = dict(
+        _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir)
+    )
+
+    _validate_config(
+        config_file,
+        expected_global_config,
+        _get_expected_efs_config_non_tls(),
+    )
+
+
+def test_write_stunnel_config_with_ipv6_and_legacy_stunnel(mocker, tmpdir):
+    ca_mocker = mocker.patch("mount_efs.add_tunnel_ca_options")
+    state_file_dir = str(tmpdir)
+
+    test_ipv6_address = "2001:db8:3333:4444:5555:6666:7777:8888"
+
+    config_file = mount_efs.write_stunnel_config_file(
+        _get_config(mocker),
+        state_file_dir,
+        FS_ID,
+        MOUNT_POINT,
+        PORT,
+        DNS_NAME,
+        VERIFY_LEVEL,
+        OCSP_ENABLED,
+        _get_mount_options_tls(),
+        DEFAULT_REGION,
+        fallback_ip_address=test_ipv6_address,
+        efs_proxy_enabled=False,
+    )
+
+    utils.assert_called_once(ca_mocker)
+
+    _validate_config(
+        config_file,
+        _get_expected_global_config(FS_ID, MOUNT_POINT, PORT, state_file_dir),
+        _get_expected_efs_config_tls(
+            dns_name=DNS_NAME,
+            fallback_ip_address=test_ipv6_address,
+            efs_proxy_enabled=False,
+        ),
     )
